@@ -9,20 +9,12 @@ import logging
 
 logger = logging.getLogger(__name__)
 
-try:
-    import pandas as pd
-    pandas_ok = True
-except ImportError:
-    class pd:
-        DataFrame = None
-        Series = None
-    pandas_ok = False
-
 
 class DuplicatedFeatures(Exception):
     pass
 
 
+# noinspection PyUnresolvedReferences
 class Condition(ABC):
     """Abstract class for Condition object. Used by Rule objets.
     A condition is a list of variable (here represented by their indexes in an array) and of conditions on those
@@ -84,7 +76,7 @@ class Condition(ABC):
         """A Condition's length is the number of features it talks about"""
         return len(self._features_indexes)
 
-    def evaluate(self, xs: Union[pd.DataFrame, np.ndarray]) -> np.ndarray:
+    def evaluate(self, xs: Union["pd.DataFrame", np.ndarray]) -> np.ndarray:
         """
         To be implemented in daughter class.
         Evaluates where a condition if fullfilled. In this abstract class that does not have any acutal condition,
@@ -108,6 +100,7 @@ class Condition(ABC):
         self._features_indexes = list(range(len(self.features_indexes)))
 
 
+# noinspection PyUnresolvedReferences
 class HyperrectangleCondition(Condition):
     """Condition class for Hyper Rectangle conditions.
 
@@ -393,12 +386,12 @@ class HyperrectangleCondition(Condition):
 
     def sort(self):
         if len(self) > 1:
-            if HyperrectangleCondition.SORT_ACCORDING_TO == "index":
+            if self.__class__.SORT_ACCORDING_TO == "index":
                 self._bmins = [x for _, x in sorted(zip(self._features_indexes, self._bmins))]
                 self._bmaxs = [x for _, x in sorted(zip(self._features_indexes, self._bmaxs))]
                 self._features_names = [x for _, x in sorted(zip(self._features_indexes, self._features_names))]
                 self._features_indexes = sorted(self._features_indexes)
-            elif HyperrectangleCondition.SORT_ACCORDING_TO == "name":
+            elif self.__class__.SORT_ACCORDING_TO == "name":
                 self._bmins = [x for _, x in sorted(zip(self._features_names, self._bmins))]
                 self._bmaxs = [x for _, x in sorted(zip(self._features_names, self._bmaxs))]
                 self._features_indexes = [x for _, x in sorted(zip(self._features_names, self._features_indexes))]
@@ -406,10 +399,10 @@ class HyperrectangleCondition(Condition):
             else:
                 raise ValueError(
                     "HyperrectangleCondition's SORT_ACCORDING_TO"
-                    f" can be 'index' or 'name', not {HyperrectangleCondition.SORT_ACCORDING_TO}"
+                    f" can be 'index' or 'name', not {self.__class__.SORT_ACCORDING_TO}"
                 )
 
-    def evaluate(self, xs: Union[pd.DataFrame, np.ndarray]) -> np.ndarray:
+    def evaluate(self, xs: Union["pd.DataFrame", np.ndarray]) -> np.ndarray:
         """
         Evaluates where a condition if fullfilled, by returning a vector of the form [0, 1, 0, 0, ...]
 
@@ -449,9 +442,7 @@ class HyperrectangleCondition(Condition):
                 leq_min &= np.less_equal(xs[:, j], self._bmaxs[i])
                 not_nan &= np.isfinite(xs[:, j])
             activation = geq_min & leq_min & not_nan
-        elif pandas_ok:
-            if not isinstance(xs, pd.DataFrame):
-                raise TypeError("xs should be a np.ndarray or a pd.DataFrame object")
+        else:
             if any([i not in xs.columns for i in self.features_names]):
                 raise IndexError("Some features names in self were not in xs DataFrame columns")
             geq_min = leq_min = not_nan = np.ones(xs.shape[0], dtype=np.ubyte)
@@ -460,8 +451,5 @@ class HyperrectangleCondition(Condition):
                 leq_min &= np.less_equal(xs[n], self._bmaxs[i])
                 not_nan &= np.isfinite(xs[n])
             activation = (geq_min & leq_min & not_nan).values
-        else:
-            raise ImportError("If xs is not a np.ndarray, Condition expects it to be a pd.DataFrame, but pandas is not "
-                              "available. Please run\npip install pandas")
 
         return activation
