@@ -44,9 +44,12 @@ class Rule(ABC):
     """Thresholds that the Rule must meet to be good. See `ruleskit.thresholds.Thresholds` for more details."""
 
     @classmethod
-    def SET_THRESHOLDS(cls, path: Union[str, Path, "TransparentPath"], show=False):
+    def SET_THRESHOLDS(cls, path: Union[str, Path, "TransparentPath", None], show=False):
         """Set thresholds globally for all futur Rules"""
-        cls.THRESHOLDS = Thresholds(path, show)
+        if path is None:
+            cls.THRESHOLDS = None
+        else:
+            cls.THRESHOLDS = Thresholds(path, show)
 
     def __init__(
         self, condition: Optional[Condition] = None, activation: Optional[Activation] = None,
@@ -61,9 +64,9 @@ class Rule(ABC):
 
         self._condition = condition
         self._activation = activation
-        self._thresholds = Rule.THRESHOLDS
-        if self._activation is not None:
-            self.check_thresholds("coverage")
+        self._thresholds = self.__class__.THRESHOLDS
+        self._good = True
+        self._bad_because = None
 
         self._coverage = None
         self._prediction = None
@@ -71,13 +74,15 @@ class Rule(ABC):
         self._time_fit = -1
         self._time_calc_activation = -1
         self._time_predict = -1
-
-        self._good = True
-        self._bad_because = None
+        if self._activation is not None:
+            self.check_thresholds("coverage")
 
     def set_thresholds(self, path: Union[str, Path, "TransparentPath"], show=False):
         """Set thresholds for this rule only"""
-        self._thresholds = Thresholds(path, show)
+        if path is None:
+            cls.THRESHOLDS = None
+        else:
+            self._thresholds = Thresholds(path, show)
 
     def check_thresholds(self, attribute: Optional[str] = None) -> None:
         """If `ruleskit.rule.Rule.THRESHOLDS` is specified, will check that this rule is good regarding those
@@ -90,11 +95,11 @@ class Rule(ABC):
             for which a threshold is defined.
         """
 
-        if Rule.THRESHOLDS is None:
+        if self.__class__.THRESHOLDS is None:
             return
 
         if attribute is not None:
-            if not Rule.THRESHOLDS(attribute, self):
+            if not self.__class__.THRESHOLDS(attribute, self):
                 self._bad_because = attribute
                 self._good = False
             return
@@ -102,7 +107,7 @@ class Rule(ABC):
         for attribute in dir(self):
             if attribute.startswith("__"):
                 continue
-            if not Rule.THRESHOLDS(attribute, self):
+            if not self.__class__.THRESHOLDS(attribute, self):
                 self._bad_because = attribute
                 self._good = False
                 return
@@ -273,7 +278,7 @@ class Rule(ABC):
         """
         arr = self._condition.evaluate(xs)
         # noinspection PyTypeChecker
-        a = Activation(arr, to_file=Rule.LOCAL_ACTIVATION)
+        a = Activation(arr, to_file=self.__class__.LOCAL_ACTIVATION)
         return a
 
     # noinspection PyUnusedLocal
@@ -418,7 +423,7 @@ class RegressionRule(Rule):
         """
         self.calc_prediction(y)
         self.calc_std(y)
-        prediction_vector = self.prediction * self.activation
+        prediction_vector = self.prediction * np.where(self.activation == 0, np.nan, self.activation)
         self.calc_criterion(prediction_vector, y, **kwargs)
 
     def calc_prediction(self, y: [np.ndarray, "pd.Series"]):
@@ -487,9 +492,12 @@ class ClassificationRule(Rule):
         self._time_calc_prediction = -1
 
     @property
-    def prediction(self) -> Union[int, str, None]:
+    def prediction(self) -> Union[int, str, np.integer, np.float, None]:
+        """Returns the rule prediction. If rule was fitted alone, the self._prediction should be a np.ndarray
+        containing the probability of each class. In that case, the most probable class is returned. If the rule was
+        fitted in a stacked fit, then the prediction is already the most probable class and it is just returned."""
         if self._prediction is not None:
-            if isinstance(self._prediction, (float, int, str)):
+            if isinstance(self._prediction, (float, int, str, np.integer, np.float)):
                 return self._prediction
             prop = [p[1] for p in self._prediction]
             idx = prop.index(max(prop))
@@ -505,9 +513,6 @@ class ClassificationRule(Rule):
         """
         Parameters
         ----------
-        xs: Union[pd:DataFrame, np.ndarray]
-            The features on which the check whether the rule is activated or not. Must be a 2-D np.ndarray
-            or pd:DataFrame.
         y: Union[np.ndarray, pd.Series]
             The targets on which to evaluate the rule prediction, and possibly other criteria. Must be a 1-D np.ndarray
             or pd.Series.
@@ -528,7 +533,7 @@ class ClassificationRule(Rule):
         t0 = time()
         if self.activation is None:
             raise ValueError("The activation vector has not been computed yet.")
-        self._prediction = functions.most_common_class(self.activation, y)
+        self._prediction = functions.class_probabilities(self.activation, y)
         self._time_calc_prediction = time() - t0
         self.check_thresholds("prediction")
 
