@@ -11,7 +11,9 @@ from .condition import HyperrectangleCondition
 
 from .rule import Rule, ClassificationRule, RegressionRule
 from .activation import Activation
+from .utils import rfunctions as functions
 import logging
+import warnings
 
 logger = logging.getLogger(__name__)
 
@@ -94,7 +96,7 @@ class RuleSet(ABC):
             if names_available:
                 self.features_names = list(set(itertools.chain(*[rule.features_names for rule in self])))
             self.set_features_indexes()
-        if RuleSet.CHECK_DUPLICATED:
+        if self.__class__.CHECK_DUPLICATED:
             self.check_duplicated_rules(self.rules, name_or_index="name" if len(self.features_names) > 0 else "index")
 
     # noinspection PyProtectedMember,PyTypeChecker
@@ -155,13 +157,13 @@ class RuleSet(ABC):
         return self.rules.__getitem__(key)
 
     def __str__(self):
-        if len(self) < 2 * RuleSet.NLINES:
+        if len(self) < 2 * self.__class__.NLINES:
             return "\n".join([str(self[i]) for i in range(len(self))])
         else:
             return "\n".join(
-                [str(self[i]) for i in range(RuleSet.NLINES)]
+                [str(self[i]) for i in range(self.__class__.NLINES)]
                 + ["..."]
-                + [str(self[i]) for i in range(len(self) - RuleSet.NLINES, len(self))]
+                + [str(self[i]) for i in range(len(self) - self.__class__.NLINES, len(self))]
             )
 
     def __hash__(self) -> hash:
@@ -259,7 +261,7 @@ class RuleSet(ABC):
             try:
                 import pandas as pd
             except ImportError:
-                raise ImportError("RuleSet's stacked activations requied pandas. Please run\npip install pandas")
+                raise ImportError("RuleSet's stacked activations requires pandas. Please run\npip install pandas")
 
             if self.stacked_activations is None:
                 if isinstance(other, Rule):
@@ -275,15 +277,15 @@ class RuleSet(ABC):
                     self.stacked_activations = pd.concat([self.stacked_activations, other.stacked_activations], axis=1)
 
     def set_features_indexes(self):
-        if len(RuleSet.all_features_indexes) > 0:
-            self.features_indexes = [RuleSet.all_features_indexes[f] for f in self.features_names]
+        if len(self.__class__.all_features_indexes) > 0:
+            self.features_indexes = [self.__class__.all_features_indexes[f] for f in self.features_names]
             for r in self._rules:
                 # noinspection PyProtectedMember
-                r._condition._features_indexes = [RuleSet.all_features_indexes[f] for f in r.features_names]
+                r._condition._features_indexes = [self.__class__.all_features_indexes[f] for f in r.features_names]
         else:
             list(set(itertools.chain(*[rule.features_indexes for rule in self])))
 
-    # noinspection PyProtectedMember
+    # noinspection PyProtectedMember,PyUnresolvedReferences
     def fit(
         self, y: Union[np.ndarray, "pd.Series"], xs: Optional[Union["pd.DataFrame", np.ndarray]] = None
     ) -> List[Rule]:
@@ -291,7 +293,7 @@ class RuleSet(ABC):
             logger.debug("Ruleset is empty. Nothing to fit.")
             return []
 
-        if RuleSet.STACKED_FIT:
+        if self.__class__.STACKED_FIT:
             try:
                 import pandas as pd
             except ImportError:
@@ -309,27 +311,30 @@ class RuleSet(ABC):
                 self.compute_stacked_activation()
                 self.stack_activation = not clean_activation
 
-            if isinstance(y, np.ndarray) and not len(self.stacked_activations.index) == y.shape[0]:
-                raise IndexError(
-                    "Stacked activation and y have different number of rows. Use pd.Series for y to"
-                    " reindex stacked activations automatically."
-                )
+            # noinspection PyUnresolvedReferences
+            if isinstance(y, np.ndarray):
+                if not len(self.stacked_activations.index) == y.shape[0]:
+                    raise IndexError(
+                        "Stacked activation and y have different number of rows. Use pd.Series for y to"
+                        " reindex stacked activations automatically."
+                    )
             else:
                 self.stacked_activations.index = y.index
-            pred, prediction_vector = self.calc_prediction(y)
+            predictions = self.calc_predictions(y)
+            criterion = self.calc_criterions(predictions, y)
+            to_drop = []
 
             if clean_activation:
                 self.del_stacked_activations()
             else:
-                self.stacked_activations.index = pd.RangeIndex(len(y.index))
-
-            prediction_vector.index = self.stack_activations.index
-            criterion = self.calc_criterion(prediction_vector, y)
-            to_drop = []
+                if isinstance(y, np.ndarray):
+                    self.stacked_activations.index = pd.RangeIndex(y.shape[0])
+                else:
+                    self.stacked_activations.index = pd.RangeIndex(len(y.index))
 
             for ir in range(len(self)):
-                self._rules[ir]._criterion = criterion.iloc[ir] if isinstance(criterion, pd.Series) else criterion
-                self._rules[ir]._prediction = pred.iloc[ir] if isinstance(pred, pd.Series) else pred
+                self._rules[ir]._criterion = criterion.iloc[ir]
+                self._rules[ir]._prediction = predictions.iloc[ir]
 
                 self._rules[ir].check_thresholds("prediction")
                 if not self._rules[ir].good:
@@ -347,6 +352,14 @@ class RuleSet(ABC):
             self.del_stacked_activations()
             for r in rules:
                 self.append(r)
+
+        if self.remember_activation:
+            self._activation = None
+            self.compute_self_activation()
+        if self.stack_activation:
+            self.stacked_activations = None
+            self.compute_stacked_activation()
+
         return to_drop
 
     def append(self, rule: Rule, update_activation: bool = True):
@@ -475,6 +488,7 @@ class RuleSet(ABC):
             del self.stacked_activations
             self.stacked_activations = None
 
+    # noinspection PyUnresolvedReferences
     def evaluate(self, xs: Union["pd.DataFrame", np.ndarray]) -> Activation:
         """Computes and returns the activation vector from an array of features.
 
@@ -493,6 +507,7 @@ class RuleSet(ABC):
         activations = [rule.evaluate(xs) for rule in self.rules]
         return Activation.multi_logical_or(activations)
 
+    # noinspection PyUnresolvedReferences
     def calc_activation(self, xs: Union[np.ndarray, "pd.DataFrame"]):
         """Uses input xs features data to compute the activation vector of all rules in self, and updates self's
         activation if self.remember_activation is True and stacked activation if self.stack_activation is True"""
@@ -565,13 +580,13 @@ class RuleSet(ABC):
                 pd.DataFrame().to_csv(path)
             return
 
-        idx = copy(RuleSet.condition_index)
+        idx = copy(self.__class__.condition_index)
         if self.rule_type == ClassificationRule:
-            idx += RuleSet.classification_rule_index
+            idx += self.__class__.classification_rule_index
         elif self.rule_type == RegressionRule:
-            idx += RuleSet.regression_rule_index
+            idx += self.__class__.regression_rule_index
         elif self.rule_type == Rule:
-            idx += RuleSet.rule_index
+            idx += self.__class__.rule_index
         else:
             raise TypeError(f"Impossible to save rule type {self.rule_type}")
 
@@ -591,7 +606,7 @@ class RuleSet(ABC):
         else:
             df.to_csv(path)
 
-    # noinspection PyProtectedMember
+    # noinspection PyProtectedMember,PyUnresolvedReferences
     @staticmethod
     def rule_to_series(irule: Tuple[int, Rule], index: list) -> "pd.Series":
         try:
@@ -604,26 +619,26 @@ class RuleSet(ABC):
         sr = pd.Series(data=[str(getattr(rule, ind)) for ind in index], name=name, index=index, dtype=str)
         return sr
 
-    @staticmethod
-    def series_to_rule(srule: "pd.Series") -> Rule:
+    # noinspection PyUnresolvedReferences
+    def series_to_rule(self, srule: "pd.Series") -> Rule:
 
-        condition_index = {c: None for c in RuleSet.condition_index}
+        condition_index = {c: None for c in self.__class__.condition_index}
 
         for ind in srule.index:
             if ind == "Unnamed: 0":
                 continue
-            if ind not in RuleSet.classification_rule_index + RuleSet.condition_index:
+            if ind not in self.__class__.classification_rule_index + self.__class__.condition_index:
                 raise IndexError(f"Invalid rule attribute '{ind}'")
 
         if "std" in srule.index:
             rule = RegressionRule()
-            rule_idx = copy(RuleSet.regression_rule_index)
+            rule_idx = copy(self.__class__.regression_rule_index)
         elif "criterion" in srule.index:
             rule = ClassificationRule()
-            rule_idx = copy(RuleSet.classification_rule_index)
+            rule_idx = copy(self.__class__.classification_rule_index)
         else:
             rule = Rule()
-            rule_idx = copy(RuleSet.rule_index)
+            rule_idx = copy(self.__class__.rule_index)
 
         for rule_ind in srule.index:
             str_value = str(srule[rule_ind])
@@ -651,6 +666,73 @@ class RuleSet(ABC):
         rule._condition = HyperrectangleCondition(**condition_index)
         return rule
 
+    # noinspection PyUnresolvedReferences
+    def calc_predictions(self, y: [np.ndarray, "pd.Series"]) -> "pd.Series":
+        """
+        Will compute the prediction of each rule in the ruleset
+
+        Parameters
+        ----------
+        y: [np.ndarray, pd.Series]
+          The targets on which to evaluate the rules predictions, and possibly other criteria. Must be a 1-D np.ndarray
+          or pd.Series.
+
+        Returns
+        -------
+        "pd.Series"
+            A pd.Series with the rules as index and the values being the predictions (most probable class in case of
+            classification).
+        """
+        try:
+            import pandas as pd
+        except ImportError:
+            raise ImportError("RuleSet's calc_predictions requires pandas. Please run\npip install pandas")
+
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", category=RuntimeWarning)
+            if len(self) == 0:
+                if self.rule_type == ClassificationRule:
+                    return pd.DataFrame()
+                else:
+                    return pd.Series()
+
+            if self.stacked_activations is None:
+                raise ValueError("The stacked activation vectors of this ruleset has not been computed yet.")
+            if self.rule_type == ClassificationRule:
+                class_probabilities = functions.class_probabilities(self.stacked_activations, y)
+                maxs = class_probabilities.max()
+                return class_probabilities[class_probabilities == maxs].apply(
+                    lambda x: x.dropna().sort_index().index[0]
+                )
+            else:
+                return functions.conditional_mean(self.stacked_activations, y)
+
+    # noinspection PyUnresolvedReferences
+    def calc_criterions(self, p: "pd.Series", y: Union[np.ndarray, "pd.Series"], **kwargs) -> "pd.Series":
+        """
+        Will compute the criterion of each rule in the ruleset
+
+        Parameters
+        ----------
+        p: "pd.Series"
+          Prediction of each rules
+        y: [np.ndarray, pd.Series]
+          The targets on which to evaluate the rules predictions, and possibly other criteria. Must be a 1-D np.ndarray
+          or pd.Series.
+
+        Returns
+        -------
+        pd.Series
+            Criterion values a set of rules (pd.Series)
+
+        """
+        if self.stacked_activations is None:
+            raise ValueError("The stacked activation vectors of this ruleset has not been computed yet.")
+        if self.rule_type == ClassificationRule:
+            return functions.calc_classification_criterion(self.stacked_activations, p, y, **kwargs)
+        else:
+            return functions.calc_regression_criterion(self.stacked_activations.replace(0, np.nan) * p, y, **kwargs)
+
 
 def traverse(o, tree_types=(list, tuple, RuleSet)):
     """Yields each elementary elements from nested list or tuple
